;#######################################################################################################
; delay_routines.asm 
;
; written by Kai Scheddin
; (C) 20.06.2007
; webmaster@scheddin.de
;
; CAUTION:
; optimized for C=ATMEGA32, XTAL=16Mhz
;
;
;
;#######################################################################################################
#ifndef _DELAY_ROUTINES_ASM_
#define _DELAY_ROUTINES_ASM_

;#######################################################################################################
;Routine Delay 50s
;
;
; CALCULATION of DELAY ALGORITHM for 50s and 16Mhz Clock:
; time_of_one_cycle=1/XTAL
; 50s=50*10^(-6)s
; number_of_cycles_for_50us=50*10^(-6)s/time_of_one_cycle (or: 50*10^(-6)*XTAL)
; for XTAL=16MHz => number_of_cycles_for_50us=800=0x0320
; factoring => 4*200=800=0x04 * 0xC8
; rcall= 3 cycles, 
; call= 4 cycles
; ldi= 1 cycle
; dec= 1 cycle
; brne= 1 cycle if true (equal), 2 cycles if false (not equal)
; ret= 4 cycles
; 
;
;number_of_delay_cycles=rcall +ldi_temp1 + temp1*((dec_temp2+brne_temp2_false)*temp2 + brne_temp2_true+dec_temp1+ldi_temp2+brne_temp1_false)+brne_temp1_true+ret
;constant_numbers=rcall+ldi_temp1+brne_temp1_true+ret
;=> constant_numbers=3+1+1+4=9 cycles
;=> number_of_delay_cycles= 9+temp1*((dec_temp2+brne_temp2_false)*temp2 + brne_temp2_true+dec_temp1+ldi_temp2+brne_temp1_false)
;=> number_of_delay_cycles= 9+temp1*(3*temp2+1+1+1+2)
;=> number_of_delay_cycles= 9+temp1*(3*temp2+5)
;=>791=temp1*(3*temp2+5)
;=>791/temp1=3*temp2+5
;=>((791/temp1)-5))/3=temp2
;=>temp2=64,25 ~ 65=0x41
;#######################################################################################################





delay50us:                              ; 50us Pause
            ldi  temp1,0x04
delay50us1_:ldi  temp2, 0x41
delay50us2_:dec  temp2
            brne delay50us2_
            dec temp1
			brne delay50us1_
			ret                          ; wieder zurck

;#######################################################################################################

;#######################################################################################################

;#######################################################################################################
;Routine delayca25us
;#######################################################################################################



delayca25us:                              ; 50us Pause
            ldi  temp1,0x02
delayca25us1_:ldi  temp2, 0x41
delayca25us2_:dec  temp2
            brne delayca25us2_
            dec temp1
			brne delayca25us1_
			ret                          ; wieder zurck

;#######################################################################################################

;#######################################################################################################



;#######################################################################################################
;Routine Delay 5ms
;
;
; CALCULATION of DELAY ALGORITHM for 5ms and 16Mhz Clock:
; time_of_one_cycle=1/XTAL
; 5ms=5*10^(-)s
; number_of_cycles_for_5ms=5*10^(-3)s/time_of_one_cycle (or: 5*10^(-3)*XTAL)
; for XTAL=16MHz => number_of_cycles_for_5ms=80000=0x013880
; factoring => 40*100*200=80000=0x04 * 0x64 *0xc8
; rcall= 3 cycles, 
; call= 4 cycles
; ldi= 1 cycle
; dec= 1 cycle
; brne= 1 cycle if true (equal), 2 cycles if false (not equal)
; ret= 4 cycles
; 
;
;number_of_delay_cycles_1=ldi_temp2 + temp2*((dec_temp3+brne_temp3_false)*temp3 + brne_temp3_true+dec_temp2+ldi_temp3+brne_temp2_false)+brne_temp2_true
;constant_numbers_sub=ldi_temp2+brne_temp3_true
;=> constant_numbers=1+1=2 cycles
;=> number_of_delay_cycles_1= 2+temp2*((dec_temp3+brne_temp3_false)*temp3 + brne_temp3_true+dec_temp2+ldi_temp3+brne_temp2_false)
;=> number_of_delay_cycles_1= 2+temp2*(3*temp3+1+1+1+2)
;=> number_of_delay_cycles_1= 2+temp2*(3*temp3+5)
;=> number_of_delay_cycles = rcall+ldi_temp1+(number_of_delay_cycles_1+dec_temp1+brne_temp1_false)*temp1+brne_temp1_true +ret
;=> number_of_delay_cycles = 3+1+(3+number_of_delay_cycles_1)*temp1+1+4
;=> number_of_delay_cycles = 9 + (3+2+temp2*(3*temp3+5))*temp1
;=> ((80000-9)/temp1)-3 = 2+temp2(3*temp3+5) |set temp1=0x04
;=> number_of_delay_cycles_1=19994,75
;=> (((19994,75-2)/temp2)-5)/3=temp3  |set temp2=0x64
;=> temp3=64,975833333 ~ 65=0x41

;ok noch ne abweichung von rund 560 cycles also ~ 0,7%
;#######################################################################################################


delay5ms:                               ; 5ms Pause

            ldi temp1,0x04
delay5ms_1: ldi temp2,0x67
delay5ms_2: ldi temp3,0x41
delay5ms_3: dec temp3
            brne delay5ms_3
			dec temp2
			brne delay5ms_2
			dec temp1
			brne delay5ms_1
           ret                              ; wieder zurck
;#######################################################################################################

;#######################################################################################################




#endif
;#######################################################################################################
;
;#######################################################################################################
