
;#######################################################################################################
; lcd_routines.asm for Displaytech 161A
;
; written by Kai Scheddin
; (C) 20.06.2007
; webmaster@scheddin.de
;
; CAUTION:
; optimized for C=ATMEGA32, XTAL=16Mhz
;
;
;
;#######################################################################################################
#ifndef _DISPLAYTECH_A161_ASM_
#define _DISPLAYTECH_A161_ASM_


;#######################################################################################################
;Routine lcd_data_trans_l
; uses specific delay routine
; and sets automaticly the right addressroom
;#######################################################################################################
lcd_data_trans_l:
           ldi temp1,0x00
		   out lcd_port,temp1
		   rcall delay50us
		   
           cpi lcd_pos_counter,8
		   brlt low_pos_l
		   
high_pos_l:mov temp2,lcd_pos_counter
		   subi temp2,8
           ldi temp1,0x40
		   add temp1,temp2
           ori temp1,0b10000000
		   out lcd_port,temp1
		   rcall lcd_enable
		   rjmp pos_set_l

low_pos_l: ldi temp1,0b10000000
		   or temp1,lcd_pos_counter
		   out lcd_port,temp1           
           rcall lcd_enable

pos_set_l: rcall delay50us
		   sbi lcd_ctrl,lcd_RS                ;set Enable
		   sbi lcd_ctrl,lcd_E                 ;set RS
		   out lcd_port,lcd_data              ;set Port with Data
		   nop                                ;wait 7 cycles (normaly 6, +1 to be sure)
		   nop
		   nop
		   nop
		   nop
		   nop
		   nop
		   cbi lcd_ctrl,lcd_E                 ;clear Enable
		   cbi lcd_ctrl,lcd_RS                ;clear RS
		   cpi lcd_pos_counter,16
		   brge l_pos_too_high
		   inc lcd_pos_counter
           ret                          	  ;return
		   l_pos_too_high:
		   ldi lcd_pos_counter,0
		   ret
;#######################################################################################################

;#######################################################################################################

 
;#######################################################################################################
;Routine lcd_data_trans_r
; uses specific delay routine
; and sets automaticly the right addressroom
;#######################################################################################################
lcd_data_trans_r:
           ldi temp1,0x00
		   out lcd_port,temp1
		   rcall delay50us
		   
           cpi lcd_pos_counter,8
		   brlt low_pos_r
		   
high_pos_r:mov temp2,lcd_pos_counter
		   subi temp2,8
           ldi temp1,0x40
		   add temp1,temp2
           ori temp1,0b10000000
		   out lcd_port,temp1
		   rcall lcd_enable
		   rjmp pos_set_r

low_pos_r: ldi temp1,0b10000000
		   or temp1,lcd_pos_counter
		   out lcd_port,temp1           
           rcall lcd_enable

pos_set_r: rcall delay50us
		   sbi lcd_ctrl,lcd_RS                ;set Enable
		   sbi lcd_ctrl,lcd_E                 ;set RS
		   out lcd_port,lcd_data              ;set Port with Data
		   nop                                ;wait 7 cycles (normaly 6, +1 to be sure)
		   nop
		   nop
		   nop
		   nop
		   nop
		   nop
		   cbi lcd_ctrl,lcd_E                 ;clear Enable
		   cbi lcd_ctrl,lcd_RS                ;clear RS
		   inc lcd_pos_counter
           ret                          	  ;return
;#######################################################################################################

;#######################################################################################################

 
;#######################################################################################################
;Routine lcd_enable
;Sets enable impuls for minimum 6 cycles, +1 to be sure
;#######################################################################################################
lcd_enable:
           sbi lcd_ctrl,lcd_E                ;Set Enable
           nop                               ;wait 7 cycles (normaly 6, +1 to be sure)
           nop
           nop
		   nop
		   nop
		   nop
		   nop
           cbi lcd_ctrl,lcd_E                 ;clear Enable
           ret                                ;return


;#######################################################################################################

;#######################################################################################################















 
;#######################################################################################################
;Routine lcd_init
;CATUTION:
;has to be called DIRECTLY after power up
;#######################################################################################################

lcd_init:
		  
           rcall  delay5ms
           ldi lcd_data, 0b00111000        ; muss 3mal hintereinander gesendet
           out lcd_port,lcd_data             ; werden zur Initialisierung
           rcall lcd_enable             ; 1
           rcall delay5ms
           rcall lcd_enable             ; 2
           rcall delay50us
		   rcall delay50us
           rcall lcd_enable             ; und 3!
		   rcall waitbfi        
		             
		   ldi lcd_data, 0b00001000        ; display off
           out lcd_port, lcd_data
		   rcall lcd_enable
           

           ldi lcd_data, 0b00000001        ;display clear
		   out lcd_port,lcd_data
		   rcall lcd_enable
           

           ldi lcd_data, 0b00000110        ;entry mode set,inc, shift
		   out lcd_port,lcd_data
		   rcall lcd_enable
           
		   rcall delay50us
		   ldi lcd_data, 0b00001100        ; display on
           out lcd_port, lcd_data
		   rcall lcd_enable
           
		   call delay50us
		   ldi lcd_data, 0b10000000
           out lcd_port,lcd_data
		   call lcd_enable
		   call delay50us

		   ldi lcd_data, 0b00000001        ;display clear
		   out lcd_port,lcd_data
		   call lcd_enable

           call delay50us
		   ldi lcd_data, 0b00000001        ;display clear
		   out lcd_port,lcd_data
		   call lcd_enable
           call delay5ms

           ret


;#######################################################################################################

;#######################################################################################################


;#######################################################################################################
;Routine lcd_reset
;CAUTION:
;works only if lcd_pwr pin is connected to Vcc of display
;#######################################################################################################
lcd_reset:
  cbi lcd_ctrl,lcd_pwr
  nop
  nop
  nop
  nop
  nop
  nop
  sbi lcd_ctrl,lcd_pwr
  rcall lcd_init
 ret
;#######################################################################################################

;#######################################################################################################



;#######################################################################################################
;Routine waitbfi
; schould check Busy Flag of Display
;  but don't work realy
;#######################################################################################################

waitbfi:
 		   ldi temp1,0x00
		   out lcd_ddr,temp1
           sbi lcd_ctrl,lcd_RW
		   sbi lcd_ctrl,lcd_E
		   nop
		   nop
		  waitbf:
		    in temp1,lcd_port
			sbrc temp1,7
			rjmp waitbf
          cbi lcd_ctrl,lcd_RW
          cbi lcd_ctrl,lcd_E
		  ldi temp2,0xff
		  out lcd_ddr,temp2
		  ret


;#######################################################################################################

;#######################################################################################################

;#######################################################################################################
;Routine write_cgram (im moment nur testroutine)
;writes the layer_main buffer into the cgram
;#######################################################################################################
write_cgram:

   
    call delayca25us
    call SM_transpose
   
  
   ldi lcd_data,lcd_cgram_addr
   out lcd_port,lcd_data
   rcall lcd_enable
   
   ldi r31,high(bw2)
   ldi r30,low(bw2)
   
   ldi temp3,8
write_cgram_loop:
   call delay50us
   ld lcd_data,z+
   out lcd_port,lcd_data
   sbi lcd_ctrl,lcd_RS
   sbi lcd_ctrl,lcd_E
   nop
   nop
   nop
   nop
   nop
   nop
   nop
   cbi lcd_ctrl,lcd_E
   cbi lcd_ctrl,lcd_RS
   dec temp3
   brne write_cgram_loop
   
   rcall delay50us
   
   ldi lcd_data,lcd_ddram_addr
   out lcd_port,lcd_data
   rcall lcd_enable
   rcall delay50us
   ret
;#######################################################################################################

;#######################################################################################################





#endif
;#######################################################################################################
; lcd_routines.asm for Displaytech 161A
;
; written by Kai Scheddin
; (C) 2007
; webmaster@scheddin.de
;
; PRECAUTION:
; For another Clock speed then XTAL=16MHz, following changes has to be made:
; TDL:
; (1) the 7 NOP Cycles @ LABELS lcd_data_trans_counter_ok, lcd_enable
; to Enable_pulse_width*XTAL=number_of_nop_cycles +1 (+1 to be sure min reached)
; (2) the delay routines has to be changed
;
;
;#######################################################################################################

